HistoricalAchievementCredit = {
	name = "HistoricalAchievementCredit",
	version = "1.1.1",

	defaults = {
		scope = 2,
		datesFirst = false,
		hideTimes = false,
	},
}

local HAC = HistoricalAchievementCredit
local LAA = LibAchievementsArchive
local LCCC = LibCodesCommonCode

local OFFSET_TOP = 8
local OFFSET_INTERNAL = 4 -- Matches the offset in HistoricalAchievementCredit.xml

local MyCharId = GetCurrentCharacterId()

local function GetCharacterIds( )
	-- 1: Current character
	-- 2: Current account
	-- 3: All accounts and servers
	if (HAC.vars.scope == 1) then
		if (LAA.GetCharacterInformation(MyCharId)) then
			return { MyCharId }
		else
			return { }
		end
	else
		return LAA.GetValidCharacterIds(HAC.vars.scope == 2)
	end
end

local function HighlightText( text, shouldHighlight )
	if (shouldHighlight) then
		return string.format("|c%s%s|r", ZO_SECOND_SELECTED_TEXT:ToHex(), text)
	else
		return text
	end
end

-- /esoui/ingame/achievements/keyboard/achievements.lua
-- Achievement:PerformExpandedLayout
local function Hook_PerformExpandedLayout( self )
	local results = { }

	-- Find the characters that have completed this achievement
	if (self.achievementId <= LAA.ArchivedGetMaxAchievementId()) then
		for _, charId in ipairs(GetCharacterIds()) do
			if (LAA.ArchivedIsAchievementComplete(charId, self.achievementId)) then
				local server, accountName, characterName = LAA.GetCharacterInformation(charId)
				table.insert(results, {
					name = characterName,
					timestamp = tonumber(Id64ToString(LAA.ArchivedGetAchievementTimestamp(charId, self.achievementId))),
					server = server,
					account = accountName,
					highlight = charId == MyCharId,
				})
			end
		end
	end

	local numberEarned = #results

	-- Sort oldest to newest
	table.sort(results, function( a, b )
		return a.timestamp < b.timestamp
	end)

	-- Handle partial progress for the current character
	if (not LAA.ArchivedIsAchievementComplete(MyCharId, self.achievementId)) then
		local server, accountName, characterName = LAA.GetCharacterInformation(MyCharId)
		if (server) then
			-- For the purposes of our calculation, each criteria is normalized to weigh equally
			local completed = 0
			local required = LAA.ArchivedGetAchievementNumCriteria(self.achievementId)
			local totalRequired = 0

			for i = 1, required do
				local _, numCompleted, numRequired = LAA.ArchivedGetAchievementCriterion(MyCharId, self.achievementId, i)
				completed = completed + numCompleted / numRequired
				totalRequired = totalRequired + numRequired
			end

			if (completed > 0 and totalRequired > 1) then
				table.insert(results, {
					name = characterName,
					partial = completed / required,
					server = server,
					account = accountName,
					highlight = true,
				})
			end
		end
	end

	-- Add to the achievement UI
	if (#results > 0) then
		local dateFormat = HAC.vars.hideTimes and "%Y-%m-%d" or "%Y-%m-%d %H:%M"

		local names = { }
		local dates = { }
		local servers = { }
		local accounts = { }

		for _, credit in ipairs(results) do
			table.insert(names, HighlightText(credit.name, credit.highlight))
			table.insert(dates, HighlightText(credit.timestamp and os.date(dateFormat, credit.timestamp) or string.format("%d%%", 100 * credit.partial), credit.highlight))
			table.insert(servers, credit.highlight and " " or credit.server)
			table.insert(accounts, credit.highlight and " " or credit.account)
		end

		-- Show the count of characters that had earned the achievement
		local header = GetString(SI_HAC_HEADER)
		if (#results > 1) then
			header = string.format("%s |c%s(%d)|r", header, ZO_DEFAULT_TEXT:ToHex(), numberEarned)
		end

		-- Using the same calculations as those found in Achievement:PerformExpandedLayout
		local footerPad = self.title:GetTop() - self.control:GetTop()
		local yOffset = self.control:GetHeight() - footerPad + OFFSET_TOP

		-- Acquire the control
		if (not self.historical) then
			local control, key = HAC.pool:AcquireObject()
			control.key = key
			self.historical = control
		end
		local control = self.historical

		-- Handle the column swap option
		local namesControl, datesControl = control.names, control.dates
		if (HAC.vars.datesFirst) then
			namesControl, datesControl = control.dates, control.names
		end

		control.header:SetText(header)
		namesControl:SetText(table.concat(names, "\n"))
		datesControl:SetText(table.concat(dates, "\n"))

		-- Show the server and account information only for the expanded scope
		if (HAC.vars.scope == 3) then
			control.servers:SetText(table.concat(servers, "\n"))
			control.accounts:SetText(table.concat(accounts, "\n"))
			control.servers:SetHidden(false)
			control.accounts:SetHidden(false)
		else
			control.servers:SetHidden(true)
			control.accounts:SetHidden(true)
		end

		control:SetParent(self.control)
		control:SetAnchor(TOP, self.control, TOP, 0, yOffset, ANCHOR_CONSTRAINS_Y)
		control:SetAnchor(LEFT, self.description, LEFT, 0, 0, ANCHOR_CONSTRAINS_X)
		control:SetHidden(false)

		local height = control.header:GetTextHeight() + namesControl:GetTextHeight() + OFFSET_INTERNAL
		self.control:SetHeight(yOffset + height + footerPad)
	end
end

-- /esoui/ingame/achievements/keyboard/achievements.lua
-- Achievement:ReleaseSharedControls
local function Hook_ReleaseSharedControls( self )
	if (self.historical) then
		HAC.pool:ReleaseObject(self.historical.key)
		self.historical = nil
	end
end

local function OnAddOnLoaded( eventCode, addonName )
	if (addonName ~= HAC.name) then return end

	EVENT_MANAGER:UnregisterForEvent(HAC.name, EVENT_ADD_ON_LOADED)

	HAC.vars = ZO_SavedVars:NewAccountWide("HistoricalAchievementCreditSavedVariables", 1, nil, HAC.defaults, nil, "$InstallationWide")

	HAC.pool = ZO_ControlPool:New("HAC_Extension")
	HAC.pool:SetCustomFactoryBehavior(function( control )
		control.header = control:GetNamedChild("Header")
		control.names = control:GetNamedChild("Names")
		control.dates = control:GetNamedChild("Dates")
		control.servers = control:GetNamedChild("Servers")
		control.accounts = control:GetNamedChild("Accounts")
	end)

	SecurePostHook(Achievement, "PerformExpandedLayout", Hook_PerformExpandedLayout)
	SecurePostHook(Achievement, "ReleaseSharedControls", Hook_ReleaseSharedControls)

	LCCC.RunAfterInitialLoadscreen(HAC.RegisterSettingsPanel)
end

EVENT_MANAGER:RegisterForEvent(HAC.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)
